import com.intellij.openapi.components.PersistentStateComponent;
import com.intellij.openapi.components.State;
import com.intellij.openapi.components.Storage;
import com.intellij.openapi.components.StoragePathMacros;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.module.ModuleComponent;
import com.intellij.openapi.options.Configurable;
import com.intellij.openapi.options.ConfigurationException;
import com.intellij.util.xmlb.annotations.Attribute;
import org.jetbrains.annotations.Nls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

@State(
        name = "myName",
        storages = @Storage(id = "default", file = StoragePathMacros.MODULE_FILE)
)

/**
 * Testclass for the PersistentStateComponent with Module based components.
 */
public class StorageTestSettingsPage
        implements Configurable, ModuleComponent, PersistentStateComponent<StorageTestSettingsPage.StateWrapper> {
    private JComponent component;
    private JCheckBox checkBox;

    private boolean value;
    private boolean modified;

    public StorageTestSettingsPage(Module module) {
        value = true;
    }

    public void initComponent() {
        modified = false;
    }

    public void disposeComponent() {
        // TODO: insert component disposal logic here
    }

    @NotNull
    public String getComponentName() {
        return "StorageTestSettingsPage";
    }

    public void projectOpened() {
        // called when project is opened
    }

    public void projectClosed() {
        // called when project is being closed
    }

    public void moduleAdded() {
        // Invoked when the module corresponding to this component instance has been completely
        // loaded and added to the project.
    }

    @Nls
    @Override
    public String getDisplayName() {
        return "Test Settings";
    }

    @Nullable
    @Override
    public String getHelpTopic() {
        return null;
    }

    @Nullable
    @Override
    public StateWrapper getState() {
        StateWrapper wrapper = new StateWrapper();
        wrapper.value = value;
        System.out.println("StorageTestSettingsPage.getState, value = " + value);
        return wrapper;
    }

    @Override
    public void loadState(StateWrapper state) {
        this.value = state.value;
        System.out.println("StorageTestSettingsPage.loadState, value = " + value);
    }

    @Nullable
    @Override
    public JComponent createComponent() {
        return component;
    }

    @Override
    public boolean isModified() {
        return modified;
    }

    @Override
    public void apply() throws ConfigurationException {
        value = checkBox.isSelected();
        modified = false;
    }

    @Override
    public void reset() {
        for (ActionListener al : checkBox.getActionListeners()) {
            checkBox.removeActionListener(al);
        }

        checkBox.setSelected(value);

        checkBox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                modified = true;
            }
        });

        modified = false;
    }

    @Override
    public void disposeUIResources() {
    }


    public class StateWrapper {
        @Attribute("testAttribute")
        public boolean value;

        public StateWrapper() {
            value = true;
        }
    }
}